@@ -18,6 +18,8 @@
 #
 import os
 
+from types import ListType
+
 from twisted.python import log
 from twisted.internet import defer
 from twisted.enterprise import adbapi
@@ -71,15 +73,17 @@ class Plugin:
         defer.returnValue(result)
 
     @defer.inlineCallbacks
-    def postprocess(self, result):
-        if result and result.has_key('players'):
-            for player in result['players']:
-                row = yield self.db.runQuery("SELECT name FROM players WHERE id = ?", [ player[0] ])
-                player[0] = row[0][0]
-        if result and result.has_key('invited') and result['invited']:
-            invited = result['invited'];
-            for index in range(len(invited)):
-                row = yield self.db.runQuery("SELECT name FROM players WHERE id = ?", [ invited[index] ])
-                invited[index] = row[0][0]
-        defer.returnValue(result)
+    def postprocess(self, results):
+        if type(results) is ListType:
+            for result in results:
+                if result.has_key('players'):
+                    for player in result['players']:
+                        row = yield self.db.runQuery("SELECT name FROM players WHERE id = ?", [ player[0] ])
+                        player[0] = row[0][0]
+                if result.has_key('invited') and result['invited']:
+                    invited = result['invited'];
+                    for index in range(len(invited)):
+                        row = yield self.db.runQuery("SELECT name FROM players WHERE id = ?", [ invited[index] ])
+                        invited[index] = row[0][0]
+        defer.returnValue(results)
 
