@@ -19,13 +19,19 @@
     $.cardstories = {
         url: "../resource",
 
+        window: window,
+
+        location: location,
+
+        noop: function() {},
+
         error: function(error) { alert(error); },
 
         xhr_error: function(xhr, status, error) {
             $.cardstories.error(error);
         },
 
-        setTimeout: function(cb, delay) { return window.setTimeout(cb, delay); },
+        setTimeout: function(cb, delay) { return $.cardstories.window.setTimeout(cb, delay); },
 
         ajax: function(o) {
             return jQuery.ajax(o);
@@ -33,7 +39,7 @@
 
         reload: function(player_id, game_id, root) {
             var search = this.reload_link(player_id, game_id, root);
-            location.search = search;
+            $.cardstories.location.search = search;
         },
 
         permalink: function(player_id, game_id, root) {
@@ -152,7 +158,7 @@
                 $('.cardstories_text', element).text(text);
             }
             var load_text = function ($o) {
-                if ($o.val().trim().length != 0) {
+                if ($o.val().trim().length !== 0) {
                     $('.cardstories_submit').addClass('cardstories_submit_ready');
                     $('.cardstories_submit', element).unbind('click').click(function() {
                         var text = $('.cardstories_text', element).val();
@@ -168,7 +174,7 @@
                     $('.cardstories_submit', element).unbind('click');
                     $('.cardstories_submit').removeClass('cardstories_submit_ready');
                 }
-            }
+            };
             load_text($('.cardstories_text', element));
             $('.cardstories_text', element).unbind('keyup').keyup(function () {
                 load_text($(this));
@@ -206,16 +212,19 @@
               }
             };
             var query = 'modified=' + request.modified;
+            var type;
             if('player_id' in request) {
               query += '&player_id=' + request.player_id;
+              type = 'lobby';
             }
             if('game_id' in request) {
               query += '&game_id=' + request.game_id;
+              type = 'game';
             }
             $this.ajax({
               async: true,
                   timeout: $this.poll_timeout * 2,
-                  url: $this.url + '?action=poll&' + query,
+                  url: $this.url + '?action=poll&type=' + type + '&' + query,
                   type: 'GET',
                   dataType: 'json',
                   global: false,
@@ -250,12 +259,12 @@
                 $this.error(data.error);
               } else {
                 if(in_progress) {
-                  $this.lobby_in_progress(player_id, data, root);
+                  $this.lobby_in_progress(player_id, data[0], root);
                 } else {
-                  $this.lobby_finished(player_id, data, root);
+                  $this.lobby_finished(player_id, data[0], root);
                 }
                 // FIXME not activated if the list of tables is empty ???
-                $this.poll({ 'modified': data.modified, 'player_id': player_id }, root);
+                $this.poll({ 'modified': data[0].modified, 'player_id': player_id }, root);
               }
             };
             var query_in_progress;
@@ -272,7 +281,7 @@
             $this.ajax({
               async: false,
                   timeout: 30000,
-                  url: $this.url + '?action=lobby&player_id=' + player_id + '&in_progress=' + query_in_progress + '&my=' + my,
+                  url: $this.url + '?action=state&type=lobby&modified=0&player_id=' + player_id + '&in_progress=' + query_in_progress + '&my=' + my,
                   type: 'GET',
                   dataType: 'json',
                   global: false,
@@ -362,16 +371,20 @@
                     deferred = this.invitation_owner(player_id, game, root);
                 }
             } else {
-                if(game.self !== null && game.self !== undefined) {
-                    if(game.self[0] === null) {
-                        deferred = this.invitation_pick(player_id, game, root);
-                        // do not disturb a player while (s)he is picking a card
-                        poll = false;
-                    } else {
-                        deferred = this.invitation_pick_wait(player_id, game, root);
-                    }
+                if ($.query.get('anonymous')) {
+                    deferred = this.invitation_anonymous(player_id, game, root);
                 } else {
-                    deferred = this.invitation_participate(player_id, game, root);
+                    if(game.self !== null && game.self !== undefined) {
+                         if(game.self[0] === null) {
+                            deferred = this.invitation_pick(player_id, game, root);
+                            // do not disturb a player while (s)he is picking a card
+                            poll = false;
+                        } else {
+                            deferred = this.invitation_pick_wait(player_id, game, root);
+                        }
+                    } else {
+                        deferred = this.invitation_participate(player_id, game, root);
+                    }
                 }
             }
             if(poll) {
@@ -559,6 +572,13 @@
             }
         },
 
+        invitation_anonymous: function(player_id, game, root) {
+            var $this = this;
+            var element = $('.cardstories_invitation .cardstories_invitation_anonymous', root);
+            this.set_active(root, element);
+            $('.cardstories_sentence', element).text(game.sentence);
+        },
+
         vote: function(player_id, game, root) {
             var poll = true;
             var deferred;
@@ -574,7 +594,11 @@
                         deferred = this.vote_voter_wait(player_id, game, root);
                     }
                 } else {
-                    deferred = this.vote_viewer(player_id, game, root);
+                    if ($.query.get('anonymous')) {
+                        deferred = this.vote_anonymous(player_id, game, root);
+                    } else {
+                        deferred = this.vote_viewer(player_id, game, root);
+                    }
                 }
             }
             if(poll) {
@@ -630,6 +654,13 @@
             });
         },
 
+        vote_anonymous: function(player_id, game, root) {
+            var $this = this;
+            var element = $('.cardstories_vote .cardstories_vote_anonymous', root);
+            this.set_active(root, element);
+            $('.cardstories_sentence', element).text(game.sentence);
+        },
+
         vote_owner: function(player_id, game, root) {
             var $this = this;
             var element = $('.cardstories_vote .cardstories_owner', root);
@@ -735,8 +766,6 @@
                           $(this).hide();
                         }
                       });
-                  } else {
-                    $('.cardstories_voter_name', this).hide();
                   }
                   $(this).show();
                 } else {
@@ -780,13 +809,13 @@
                 if('error' in data) {
                     $this.error(data.error);
                 } else {
-                    $this[data.state](player_id, data, root);
+                    $this[data[0].state](player_id, data[0], root);
                 }
             };
             $this.ajax({
                 async: false,
                 timeout: 30000,
-                url: $this.url + '?action=game&game_id=' + game_id + '&player_id=' + player_id,
+                url: $this.url + '?action=state&type=game&modified=0&game_id=' + game_id + '&player_id=' + player_id,
                 type: 'GET',
                 dataType: 'json',
                 global: false,
@@ -873,7 +902,7 @@
     };
 
     $(window).bind("beforeunload", function() {
-        $.cardstories.error = function(error) { };
-      });
+        $.cardstories.error = $.cardstories.noop;
+    });
 
 })(jQuery);
