@@ -16,14 +16,90 @@
 //
 module("cardstories");
 
+var cardstories_default_reload = $.cardstories.reload;
+var cardstories_default_setTimeout = $.cardstories.setTimeout;
+var cardstories_default_ajax = $.cardstories.ajax;
+var cardstories_original_error = $.cardstories.error;
+var cardstories_original_poll_ignore = $.cardstories.poll_ignore;
+
 function setup() {
-    $.cardstories.setTimeout = function(cb, delay) { console.log("timeout"); return window.setTimeout(cb, delay); };
+    $.cardstories.setTimeout = function(cb, delay) { return window.setTimeout(cb, delay); };
     $.cardstories.ajax = function(o) { throw o; };
     $.cardstories.reload = $.cardstories.game_or_lobby;
     $.cardstories.confirm_participate = true;
     $.cardstories.poll_ignore = function() { throw 'poll_ignore'; };
+    $.cardstories.error = cardstories_original_error;
 }
 
+test("error", function() {
+    setup();
+    expect(1);
+
+    var alert = window.alert;
+    window.alert = function(err) { equal(err, 'an error occurred', 'calls window.alert on error'); };
+    $.cardstories.error('an error occurred');
+    window.alert = alert;
+  });
+
+test("setTimeout", function() {
+  expect(2);
+
+  $.cardstories.setTimeout = cardstories_default_setTimeout;
+
+  var setTimeout = $.cardstories.window.setTimeout;
+  $.cardstories.window.setTimeout = function(cb, delay) {
+    equal(cb, 'a function');
+    equal(delay, 42);
+  };
+
+  $.cardstories.setTimeout('a function', 42);
+  $.cardstories.window.setTimeout = setTimeout;
+});
+
+test("ajax", function() {
+  expect(2);
+
+  $.cardstories.ajax = cardstories_default_ajax;
+  var ajax = jQuery.ajax;
+  jQuery.ajax = function(options) {
+    equal(options, 'some ajax options', 'calls jQuery.ajax with the supplied options');
+    return 'some ajax result';
+  };
+
+  var result = $.cardstories.ajax('some ajax options');
+  equal(result, 'some ajax result', 'returns the result of jQuery.ajax call');
+
+  jQuery.ajax = ajax;
+});
+
+test("reload", function() {
+    expect(4);
+
+    var location = $.cardstories.location;
+    var reload_link = $.cardstories.reload_link;
+    $.cardstories.reload = cardstories_default_reload;
+    $.cardstories.location = {search: ''};
+    $.cardstories.reload_link = function(player_id, game_id, root) {
+        equal(player_id, 11);
+        equal(game_id, 101);
+        equal(root, 'the root');
+        return '?reload=link&';
+    };
+
+    $.cardstories.reload(11, 101, 'the root');
+    equal($.cardstories.location.search, '?reload=link&');
+
+    $.cardstories.location = location;
+    $.cardstories.reload_link = reload_link;
+});
+
+test("xhr_error", function() {
+    expect(1);
+
+    $.cardstories.error = function(err) { equal(err, 'an xhr error occurred', 'calls $.cardstories.error'); };
+    $.cardstories.xhr_error({xhr: 'object'}, 500, 'an xhr error occurred');
+});
+
 test("play_again_finish_state", function() {
     setup();
     expect(5);
@@ -32,7 +108,7 @@ test("play_again_finish_state", function() {
 	    'id': 7,
 	    'state': 'fake_state',
 	    'board': [],
-	    'players': [],
+	    'players': []
 	};
 	var root = $('#qunit-fixture .cardstories');
 	game.owner = false;
@@ -45,7 +121,7 @@ test("play_again_finish_state", function() {
     var send_game = $.cardstories.send_game;
     var $textarea = $('.cardstories_text', $('.cardstories_advertise', root));
     $.cookie('CARDSTORIES_INVITATIONS', null);
-    $textarea.val('aaa@aaa.aaa\nbbb@bbb.bbb\nccc@ccc.ccc')
+    $textarea.val('aaa@aaa.aaa\nbbb@bbb.bbb\nccc@ccc.ccc');
     var text = $textarea.val();
     $.cardstories.send_game = function () {}; //do nothing
     $.cardstories.advertise(player_id, game.id, root);
@@ -56,7 +132,7 @@ test("play_again_finish_state", function() {
         equal(arg_player_id, player_id);
         equal(arg_root, root);
         $.cardstories.create = create;
-    }
+    };
     $('.play_again', root).click();
     equal(text, inv_cookie);
 });
@@ -167,14 +243,14 @@ test("test confirm_results_publication", function () {
         equal(arg_game.id, game_id);
         ok($(root).hasClass('cardstories_root'), 'cardstories_root');
         $.cardstories.vote_owner = vote_owner;
-    }
+    };
 
-	var send_game = $.cardstories.send_game;
+    var send_game = $.cardstories.send_game;
     $.cardstories.send_game = function (arg_player_id, arg_game_id) {
         equal(arg_player_id, player_id);
         equal(arg_game_id, game_id);
         $.cardstories.send_game = send_game;
-    }
+    };
 
     $.cardstories.confirm_results_publication(player_id, game, root);
     $('.cardstories_notyet_announce_results').click();
@@ -207,6 +283,25 @@ test("send_game", function() {
     $.cardstories.send_game(player_id, game_id, $('#qunit-fixture .cardstories_create'), 'QUERY');
 });
 
+test("send_game on error", function() {
+    setup();
+    expect(1);
+
+    var player_id = 15;
+    var game_id = 101;
+
+    $.cardstories.ajax = function(options) {
+        var data = {error: 'error on send_game'};
+        options.success(data);
+    };
+
+    $.cardstories.error = function(err) {
+        equal(err, 'error on send_game', 'calls $.cardstories.error');
+    };
+
+    $.cardstories.send_game(player_id, game_id, $('#qunit-fixture .cardstories_create'), 'QUERY');
+});
+
 test("create", function() {
     setup();
     stop();
@@ -254,6 +349,35 @@ test("create", function() {
         });
 });
 
+test("create on error", function() {
+    setup();
+    stop();
+    expect(1);
+
+    var player_id = 15;
+
+    $.cardstories.ajax = function(options) {
+        var data = {error: 'error on create'};
+        options.success(data);
+    };
+
+    $.cardstories.error = function(err) {
+        equal(err, 'error on create', 'calls $.cardstories.error');
+    };
+
+    var element = $('#qunit-fixture .cardstories_create');
+    $.cardstories.
+        create(player_id, $('#qunit-fixture .cardstories')).
+        done(function(is_ready) {
+            var first_card = $('.cardstories_card:nth(0)', element);
+            first_card.click();
+            $('.cardstories_card_confirm_ok', element).click();
+            $('.cardstories_write_sentence .cardstories_sentence', element).val('SENTENCE');
+            $('.cardstories_write_sentence .cardstories_submit', element).submit();
+            start();
+        });
+});
+
 test("widget lobby", function() {
     setup();
     expect(4);
@@ -263,7 +387,7 @@ test("widget lobby", function() {
     ok(!$('#qunit-fixture .cardstories').hasClass('cardstories_root'), 'no cardstories_root');
     $.cardstories.ajax = function(options) {
         equal(options.type, 'GET');
-        equal(options.url, $.cardstories.url + '?action=lobby&player_id=' + player_id + '&in_progress=true&my=true');
+        equal(options.url, $.cardstories.url + '?action=state&type=lobby&modified=0&player_id=' + player_id + '&in_progress=true&my=true');
     };
     $('#qunit-fixture .cardstories').cardstories(player_id);
     ok($('#qunit-fixture .cardstories').hasClass('cardstories_root'), 'cardstories_root');
@@ -271,7 +395,7 @@ test("widget lobby", function() {
 
 test("game", function() {
     setup();
-    expect(4);
+    expect(5);
 
     var player_id = 15;
     var game_id = 101;
@@ -286,23 +410,37 @@ test("game", function() {
 
     $.cardstories.ajax = function(options) {
         equal(options.type, 'GET');
-        equal(options.url, $.cardstories.url + '?action=game&game_id=' + game_id + '&player_id=' + player_id);
+        equal(options.url, $.cardstories.url + '?action=state&type=game&modified=0&game_id=' + game_id + '&player_id=' + player_id);
 	var game = {
 	    'id': game_id,
 	    'state': 'fake_state'
 	};
-	options.success(game);
+	options.success([game]);
     };
 
     $.cardstories.game(player_id, game_id, root);
 
     // Test link to lobby
-    $.cardstories.reload = function(player_id2, game_id2, root2) {
+    $.cardstories.reload = function(player_id2) {
         equal(player_id2, player_id);
-        equal(game_id2, game_id);
-        equal(root2, root);
     };
-    $('.cardstories_game .cardstories_go_lobby', root).click();
+    $('.cardstories_go_lobby', root).click();
+});
+
+test("game on error", function() {
+    setup();
+    expect(1);
+
+    $.cardstories.ajax = function(options) {
+        var data = {error: 'error on game'};
+        options.success(data);
+    };
+
+    $.cardstories.error = function(err) {
+        equal(err, 'error on game', 'calls $.cardstories.error');
+    };
+
+    $.cardstories.game(11, 111, 'the root');
 });
 
 test("invitation_owner_invite_more", function() {
@@ -506,6 +644,46 @@ test("invitation_pick_wait", function() {
     equal($(element_pick).hasClass('cardstories_active'), true, 'pick active');
 });
 
+test("invitation_anonymous", function() {
+    setup();
+    expect(3);
+
+    var player_id = null;
+    var game_id = 101;
+    var picked = null;
+    var cards = [1,2,3,4,picked,5];
+    var sentence = 'SENTENCE';
+
+    var _query = $.query;
+
+    // Without player_id in the URL
+    $.query = {
+        'get': function(attr) {
+            if (attr == "anonymous") {
+                return "yes";
+            }
+        }
+    };
+
+    var game = {
+	'id': game_id,
+	'self': [picked, null, cards],
+	'sentence': sentence
+    };
+
+    $.cardstories.poll_ignore = function(ignored_request, ignored_answer, new_poll, old_poll) {
+        equal(ignored_request.game_id, game_id, 'poll_ignore request game_id');
+        equal(new_poll, undefined, 'poll_ignore metadata not set');
+    };
+
+    var element = $('#qunit-fixture .cardstories_invitation .cardstories_invitation_anonymous');
+    $.cardstories.invitation(player_id, game, $('#qunit-fixture .cardstories'));
+    equal($('.cardstories_sentence', element).text(), sentence);
+
+    // Restore the original query
+    $.query = _query;
+});
+
 test("invitation_participate", function() {
     setup();
     expect(11);
@@ -552,12 +730,12 @@ test("widget invitation", function() {
 
     var ajax_poll = function(options) {
         equal(options.type, 'GET');
-        equal(options.url, $.cardstories.url + '?action=poll&modified=' + modified + '&player_id=' + player_id + '&game_id=' + game_id);
+        equal(options.url, $.cardstories.url + '?action=poll&type=game&modified=' + modified + '&player_id=' + player_id + '&game_id=' + game_id);
     };
 
     $.cardstories.ajax = function(options) {
         equal(options.type, 'GET');
-        equal(options.url, $.cardstories.url + '?action=game&game_id=' + game_id + '&player_id=' + player_id);
+        equal(options.url, $.cardstories.url + '?action=state&type=game&modified=0&game_id=' + game_id + '&player_id=' + player_id);
 	var game = {
 	    'id': game_id,
 	    'state': 'invitation',
@@ -565,7 +743,7 @@ test("widget invitation", function() {
 	    'sentence': sentence
 	};
         $.cardstories.ajax = ajax_poll;
-	options.success(game);
+	options.success([game]);
 	equal($('#qunit-fixture .cardstories_participate .cardstories_sentence').text(), sentence);
     };
 
@@ -673,6 +851,48 @@ test("invitation_voter_wait", function() {
     equal($(element_pick).hasClass('cardstories_active'), true, 'voter active');
 });
 
+test("vote_anonymous", function() {
+    setup();
+    expect(3);
+
+    var player_id = null;
+    var game_id = 101;
+    var picked = null;
+    var voted = null;
+    var board = [1,2,voted,4,picked,6];
+    var sentence = 'SENTENCE';
+
+    var _query = $.query;
+
+    // Without player_id in the URL
+    $.query = {
+        'get': function(attr) {
+            if (attr == "anonymous") {
+                return "yes";
+            }
+        }
+    };
+
+    var game = {
+	'id': game_id,
+	'board': board,
+	'self': null,
+	'sentence': sentence
+    };
+
+    $.cardstories.poll_ignore = function(ignored_request, ignored_answer, new_poll, old_poll) {
+        equal(ignored_request.game_id, game_id, 'poll_ignore request game_id');
+        equal(new_poll, undefined, 'poll_ignore metadata not set');
+    };
+
+    var element = $('#qunit-fixture .cardstories_vote .cardstories_vote_anonymous');
+    $.cardstories.vote(player_id, game, $('#qunit-fixture .cardstories'));
+    equal($('.cardstories_sentence', element).text(), sentence);
+
+    // Restore the original query
+    $.query = _query;
+});
+
 test("vote_viewer", function() {
     setup();
     expect(7);
@@ -878,7 +1098,7 @@ test("results_board", function() {
 
 test("advertise", function() {
     setup();
-    expect(6);
+    expect(8);
     
     var owner_id = 15;
     var game_id = 100;
@@ -906,13 +1126,17 @@ test("advertise", function() {
     $('.cardstories_text', element).text('');
     $.cardstories.advertise(owner_id, game_id, $('#qunit-fixture .cardstories'));
 
+    // button should be enabled only when text is not blank
+    ok(!$('.cardstories_submit', element).hasClass('cardstories_submit_ready'), 'button should be disabled');
+    $('.cardstories_text', element).text('player1').keyup();
+    ok($('.cardstories_submit', element).hasClass('cardstories_submit_ready'), 'button should be enabled');
+
     // GAME_URL supplant
     var root = $('#qunit-fixture .cardstories');    
     var facebookUrl = $('#facebook_url').html().supplant({'GAME_URL': escape($.cardstories.permalink(owner_id, game_id, root))});
     var src = $('.cardstories_fb_invite', element).attr('src');
     ok(src.indexOf('GAME_URL') == -1, '{GAME_URL} supplant');
     equal(src, facebookUrl);
-    
   });
 
 test("refresh_lobby", function() {
@@ -920,6 +1144,7 @@ test("refresh_lobby", function() {
     expect(15);
 
     var in_progress;
+    var my;
     var player_id = 10;
     var game1 = 100;
     var sentence1 = 'sentence1';
@@ -929,8 +1154,8 @@ test("refresh_lobby", function() {
 
     $.cardstories.ajax = function(options) {
         equal(options.type, 'GET');
-        equal(options.url, $.cardstories.url + '?action=lobby&player_id=' + player_id + '&in_progress=' + in_progress.toString() + '&my=true');
-	options.success(games);
+        equal(options.url, $.cardstories.url + '?action=state&type=lobby&modified=0&player_id=' + player_id + '&in_progress=' + in_progress.toString() + '&my=' + my);
+	options.success([games]);
     };
 
     $.cardstories.poll_ignore = function(ignored_request, ignored_answer, new_poll, old_poll) {
@@ -940,16 +1165,36 @@ test("refresh_lobby", function() {
     };
 
     in_progress = true;
+    my = true;
     equal($('#qunit-fixture .cardstories_lobby .cardstories_in_progress.cardstories_active').length, 0, 'in_progress not active');
-    $.cardstories.refresh_lobby(player_id, in_progress, true, root);
+    $.cardstories.refresh_lobby(player_id, in_progress, my, root);
     equal($('#qunit-fixture .cardstories_lobby .cardstories_in_progress.cardstories_active').length, 1, 'in_progress active');
 
     in_progress = false;
+    my = false;
     equal($('#qunit-fixture .cardstories_lobby .cardstories_finished.cardstories_active').length, 0, 'finished not active');
-    $.cardstories.refresh_lobby(player_id, in_progress, true, root);
+    $.cardstories.refresh_lobby(player_id, in_progress, my, root);
     equal($('#qunit-fixture .cardstories_lobby .cardstories_finished.cardstories_active').length, 1, 'finished active');
     equal($('#qunit-fixture .cardstories_lobby .cardstories_in_progress.cardstories_active').length, 0, 'in_progress not active');
-  });
+});
+
+test("refresh_lobby on error", function() {
+    setup();
+    expect(1);
+
+    var root = $('#qunit-fixture .cardstories');
+
+    $.cardstories.ajax = function(options) {
+        var data = {error: 'error on refresh_lobby'};
+        options.success(data);
+    };
+
+    $.cardstories.error = function(err) {
+        equal(err, 'error on refresh_lobby', 'calls $.cardstories.error');
+    };
+
+    $.cardstories.refresh_lobby(42, true, true, root);
+});
 
 test("lobby_games", function() {
     setup();
@@ -985,7 +1230,7 @@ test("lobby_games", function() {
     equal($('.cardstories_lobby_sentence', first).metadata({type: "attr", name: "data"}).game_id, game1);
     $.cardstories.ajax = function(options) {
         equal(options.type, 'GET');
-        equal(options.url, $.cardstories.url + '?action=game&game_id=' + game1 + '&player_id=' + player_id);
+        equal(options.url, $.cardstories.url + '?action=state&type=game&modified=0&game_id=' + game1 + '&player_id=' + player_id);
     };
     $('.cardstories_lobby_sentence', first).click();
     var second = $('.cardstories_games tbody tr:nth(1)', element);
@@ -996,7 +1241,7 @@ test("lobby_games", function() {
     equal($('.cardstories_lobby_sentence', second).metadata({type: "attr", name: "data"}).game_id, game2);
     $.cardstories.ajax = function(options) {
         equal(options.type, 'GET');
-        equal(options.url, $.cardstories.url + '?action=game&game_id=' + game2 + '&player_id=' + player_id);
+        equal(options.url, $.cardstories.url + '?action=state&type=game&modified=0&game_id=' + game2 + '&player_id=' + player_id);
     };
     $('.cardstories_lobby_sentence', second).click();
     // modify the number row per page
@@ -1019,6 +1264,17 @@ test("lobby_games", function() {
     equal($('.cardstories_games tbody tr:nth(0) .cardstories_lobby_sentence', element).text(), sentence1);
   });
 
+test("lobby_games without games", function() {
+    setup();
+    expect(1);
+
+    var player_id = 10;
+    var games = {games: [], win: {}};
+    var element = $('#qunit-fixture .cardstories .cardstories_games_test');
+    $.cardstories.lobby_games(player_id, games, element, $('#qunit-fixture .cardstories'));
+    equal($('.cardstories_pager', element).css('display'), 'none', 'pager is hidden');
+});
+
 test("poll_discard", function() {
     setup();
     expect(3);
@@ -1031,7 +1287,29 @@ test("poll_discard", function() {
     $(root).metadata().poll = poll;
     equal($.cardstories.poll_discard(root), poll + 1, '++ to discard');
     equal($(root).metadata().poll, poll + 1, 'persists');
-  });
+});
+
+test("poll_ignore", function() {
+    $.cardstories.poll_ignore = cardstories_original_poll_ignore;
+
+    if (console && console.log) {
+        expect(2);
+
+        var log = console.log;
+
+        console.log = function(text) {
+            equal(text, 'poll ignored because 42 higher than 24');
+        };
+        $.cardstories.poll_ignore('req', 'answer', 42, 24);
+
+        console.log = function(text) {
+            equal(text, 'poll ignored because metadata is not set');
+        };
+        $.cardstories.poll_ignore('req', 'answer', undefined, 42);
+
+        console.log = log;
+    }
+});
 
 test("start_story", function() {
     setup();
@@ -1047,7 +1325,7 @@ test("start_story", function() {
     $.cardstories.start_story(player_id, root);
     equal($('#qunit-fixture .cardstories_create .cardstories_pick_card.cardstories_active').length, 1, 'pick_card active');
     equal($(root).metadata().poll, poll + 1);
-  });
+});
 
 test("lobby_in_progress", function() {
     setup();
@@ -1067,7 +1345,7 @@ test("lobby_in_progress", function() {
     // lobby tab
     $.cardstories.ajax = function(options) {
         equal(options.type, 'GET');
-        equal(options.url, $.cardstories.url + '?action=lobby&player_id=' + player_id + '&in_progress=false&my=true');
+        equal(options.url, $.cardstories.url + '?action=state&type=lobby&modified=0&player_id=' + player_id + '&in_progress=false&my=true');
     };
     $('.cardstories_tab_finished', element).click();
     // list of games
@@ -1104,7 +1382,7 @@ test("lobby_finished", function() {
     // lobby tab
     $.cardstories.ajax = function(options) {
         equal(options.type, 'GET');
-        equal(options.url, $.cardstories.url + '?action=lobby&player_id=' + player_id + '&in_progress=true&my=true');
+        equal(options.url, $.cardstories.url + '?action=state&type=lobby&modified=0&player_id=' + player_id + '&in_progress=true&my=true');
     };
     $('.cardstories_tab_in_progress', element).click();
     // list of games
@@ -1146,12 +1424,12 @@ test("poll", function() {
     //
     var game_ajax1 = function(options) {
         equal(options.type, 'GET');
-        equal(options.url, $.cardstories.url + '?action=game&game_id=' + game_id + '&player_id=' + player_id);
+        equal(options.url, $.cardstories.url + '?action=state&type=game&modified=0&game_id=' + game_id + '&player_id=' + player_id);
     };
 
     var poll_ajax1 = function(options) {
         equal(options.type, 'GET');
-        equal(options.url, $.cardstories.url + '?action=poll&modified=' + modified + '&player_id=' + player_id + '&game_id=' + game_id);
+        equal(options.url, $.cardstories.url + '?action=poll&type=game&modified=' + modified + '&player_id=' + player_id + '&game_id=' + game_id);
         $.cardstories.ajax = game_ajax1;
 	options.success(request);
     };
@@ -1175,7 +1453,7 @@ test("poll", function() {
     $.cardstories.ajax = function(options) {
         $.cardstories.poll_discard(root);
         equal(options.type, 'GET');
-        equal(options.url, $.cardstories.url + '?action=poll&modified=' + modified + '&player_id=' + player_id + '&game_id=' + game_id);
+        equal(options.url, $.cardstories.url + '?action=poll&type=game&modified=' + modified + '&player_id=' + player_id + '&game_id=' + game_id);
 	options.success(answer);
     };
 
@@ -1186,12 +1464,12 @@ test("poll", function() {
     //
     var poll_again = function(options) {
         equal(options.type, 'GET', 'poll again');
-        equal(options.url, $.cardstories.url + '?action=poll&modified=' + modified + '&player_id=' + player_id + '&game_id=' + game_id);
+        equal(options.url, $.cardstories.url + '?action=poll&type=game&modified=' + modified + '&player_id=' + player_id + '&game_id=' + game_id);
     };
 
     $.cardstories.ajax = function(options) {
         equal(options.type, 'GET', 'poll timeout');
-        equal(options.url, $.cardstories.url + '?action=poll&modified=' + modified + '&player_id=' + player_id + '&game_id=' + game_id);
+        equal(options.url, $.cardstories.url + '?action=poll&type=game&modified=' + modified + '&player_id=' + player_id + '&game_id=' + game_id);
         $.cardstories.ajax = poll_again;
 	options.success({'timeout': [modified+1111]});
     };
@@ -1387,8 +1665,8 @@ test("select_cards single", function() {
         var links = $('a.cardstories_card', element);
         links.each(function(index) {
             $(this).click();
-            var condition = $(this).hasClass('cardstories_card_selected') == index == 0;
-            var not = $(this).hasClass('cardstories_card_selected') ? '' : 'not'
+            var condition = $(this).hasClass('cardstories_card_selected') == index === 0;
+            var not = $(this).hasClass('cardstories_card_selected') ? '' : 'not';
             ok(condition, 'Card ' + index + ' ' + not +  ' picked.');
         });
         start();
@@ -1414,7 +1692,7 @@ test("create_deck", function() {
 
 test("credits", function() {
     setup();
-    expect(3);
+    expect(4);
 
     var root = $('#qunit-fixture .cardstories');
     var long = $('.cardstories_credits_long', root);
@@ -1423,6 +1701,8 @@ test("credits", function() {
     $('.cardstories_credits_short', root).click();
     equal(long.is(':visible'), true, 'credits visible');
     equal($('.jspArrowUp', long).css('background-image').indexOf('url('), 0, 'up arrow for scrolling');
+    $('.cardstories_credits_close', long).click();
+    equal(long.is(':visible'), false, 'credits not visible');
   });
 
 test("solo", function() {
@@ -1450,4 +1730,67 @@ test("solo", function() {
     };
 
     $.cardstories.solo(player_id, root);
-  });
+});
+
+test("solo on error", function() {
+    setup();
+    expect(1);
+
+    var root = $('#qunit-fixture .cardstories');
+    var player_id = 15;
+
+    $.cardstories.ajax = function(options) {
+        var data = {error: 'error on solo'};
+	options.success(data);
+    };
+
+    $.cardstories.error = function(err) {
+        equal(err, 'error on solo', 'calls $.cardstories.error');
+    };
+
+    $.cardstories.solo(player_id, root);
+});
+
+test("poll on error", function() {
+    setup();
+    expect(1);
+
+    var root = $('#qunit-fixture .cardstories');
+    root.metadata().poll = 1;
+
+    $.cardstories.ajax = function(options) {
+        var data = {error: 'error on poll'};
+	options.success(data);
+    };
+
+    $.cardstories.error = function(err) {
+        equal(err, 'error on poll', 'calls $.cardstories.error');
+    };
+
+    $.cardstories.poll({}, root);
+});
+
+test("trigger_keypress, trigger_keydown helpers", function() {
+    expect(4);
+
+    var key_code = 101;
+    var trigger = $.event.trigger;
+
+    $.event.trigger = function(options) {
+        equal(options.type, 'keypress');
+        equal(options.which, key_code);
+    };
+
+    $('#qunit-fixture').trigger_keypress(key_code);
+    $('#qunit-fixture').trigger_keydown(key_code);
+
+    $.event.trigger = trigger;
+});
+
+test("onbeforeunload", function() {
+    setup();
+    expect(1);
+
+    $(window).trigger('beforeunload');
+    equal($.cardstories.error, $.cardstories.noop, 'error handler gets set to a noop function on beforeunload');
+});
